package org.jeecg.modules.geoAnalysis.service;

import com.baomidou.mybatisplus.extension.service.IService;
import org.jeecg.modules.commonQueryConfig.entity.CommonQueryConfig;
import org.jeecg.modules.geoAnalysis.entity.GeoAnalysis;
import org.jeecg.modules.geoAnalysis.entity.ZYFXPZB;
import org.jeecg.modules.gis.lochistory.entity.LocHistory;

import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description: 定位信息表
 * @Author: jeecg-boot
 * @Date: 2022-07-01
 * @Version: V1.0
 */
public interface IGeoAnlysisService extends IService<GeoAnalysis> {
    ArrayList<GeoAnalysis> DrawMap(List<ZYFXPZB> zyfxpzb, String shape);

    ZYFXPZB SelectByTbs(String MTABLENAME, String atableename);

    List<ZYFXPZB> SelectByTb(String MTABLENAME);

    ArrayList<GeoAnalysis> DrawMapByShape(List<ZYFXPZB> zyfxpzb, List shape, String table);

    HashMap<String, Object> SelectDetails(HashMap map);

    ArrayList<CommonQueryConfig> SelectByTable(String TABLEALIAS);

    Boolean exportShps(HttpServletResponse response,String id);


}
